#ifndef RECEIVEBUFFER_H
#define RECEIVEBUFFER_H


#include <QObject>
#include <QByteArray>
#include <QList>
#include "com_interface.h"




class QTimer;
class VMC;
enum class RECEIVE_ERROR : quint8;


class ReceiveBuffer : public QObject
{
    Q_OBJECT


private:
    VMC *vmc;
    COM_interface *interface;

    QList<QByteArray> messageQueue;
    QByteArray currentMessage;

    int checkMessageForCRC();
    bool checkCRC(quint8 CRC, int index);

    /* the receiveIdleTimer clears the receive buffer (currentMessag), if for
     * a certain time no data was received.
     * This is for improve speed for responding on vmc messages because possible
     * wrong data is removed from queue.
     */
    QTimer *receiveIdleTimer;

    // for debug:
    void qCriticalBAMessage(QByteArray message);

public:
    explicit ReceiveBuffer(COM_interface *cinterface, QObject *parent = nullptr);
    ~ReceiveBuffer();
    
    int size();


signals:
    void CommandAvailiable(QByteArray command);
    void ReceiveError(RECEIVE_ERROR error);
    void ReceiveResponse(quint8 response);


private slots:
    void removeCommand();
    void getData(QByteArray *data);

    void onReceiveIdleTimerTimeout();
    
};


enum class RECEIVE_ERROR : quint8 {
    NO_ERROR = 0,
    CRC,
    NO_COMMAND,
    INVALID_COMMAND
};

#endif // RECEIVEBUFFER_H
