#ifndef SENDBUFFER_H
#define SENDBUFFER_H

#include <QObject>
#include <QByteArray>
#include <QQueue>
#include "com_interface.h"


class QTimer;
class VMC;
enum class SEND_ERROR : quint8;

#define SB_MAX_RESEND 3
#define SB_MAX_SENDQUEUESIZE 10


class SendBuffer : public QObject
{
    Q_OBJECT

private:
    VMC *vmc;
    COM_interface *interface;

    QQueue<QByteArray> messageQueue;
    quint8 resendCounter;

    /* timer for resending messages to vmc, if no valid response is resent from vmc
     */
    QTimer *responseTimeoutTimer;
    quint8 responseTimerTimeoutCounter;

    void PrivateSendMessage();

public:
    explicit SendBuffer(COM_interface *cinterface, QObject *parent = nullptr);
    ~SendBuffer();

    int SendMessage(QByteArray & ba, bool enqueue = false);
    
signals:
    void writeLogSent(QByteArray ba);
    void sendError(SEND_ERROR error);

public slots:
    void onReceiveResponse(quint8 response);
    void onResponseTimeoutTimerTimeout();
};


enum class SEND_ERROR : quint8 {
    NO_ERROR = 0,
    RESPONSE,
    RESPONSE_TIMER_TIMEOUT,
    MULTIPLE_RESPONSE_TIMER_TIMEOUT
};

#endif // SENDBUFFER_H
