#include "com_interface.h"
#include <QtDebug>

#if QT_VERSION >=0x050000
#include <QSerialPort>
#else
#include "qextserialport.h"
#endif

COM_interface::COM_interface(QObject *parent) :
    QObject(parent)
{
#if QT_VERSION >=0x050000
    port = new QSerialPort(this);
#else
    port = new QextSerialPort(QextSerialPort::EventDriven, this);
#endif

    buffer = new QByteArray();
}


COM_interface::~COM_interface()
{
    delete buffer;
    delete port;
}

/***************************************************************************
 * *************************************************************************
 *
 * QT5 ( = QSerialPort) specific methodes
 *
 ***************************************************************************
 ***************************************************************************/
#if QT_VERSION >=0x050000

int COM_interface::open(const QString & portname, int baudrate)
{
    QSerialPort::BaudRate br = QSerialPort::Baud115200;

    switch (baudrate) {
        case  9600: br = QSerialPort::Baud9600;
                    break;
        case 57600: br = QSerialPort::Baud57600;
                    break;
        default:    br = QSerialPort::Baud115200;
                    break;
    }

    qDebug() << "COM_interface::open(): baudrate = " << baudrate;
    qDebug() << "COM_interface::open(): br = " << br;

    port->setPortName(portname);
    port->setBaudRate(br);
    port->setStopBits(QSerialPort::OneStop);
    port->setParity(QSerialPort::NoParity);
    port->setDataBits(QSerialPort::Data8);


    if (port->open(QIODevice::ReadWrite) == true) {
        connect(port, SIGNAL(readyRead()), this, SLOT(onReadyRead()));

        qDebug() << "listening for data on" << port->portName();

        emit connected();
        return 1;
    }
    else {
        qDebug() << "device failed to open:";
        qDebug() << "               device:" << portname;
        qDebug() << "          errorstring:" << port->errorString();
        return -1;
    }
}

int COM_interface::open(const QString & portname, int baudrate, int dataBits, int stopBits, int parity)
{
    QSerialPort::BaudRate br = QSerialPort::Baud115200;
    QSerialPort::DataBits db = QSerialPort::Data8;
    QSerialPort::StopBits sb = QSerialPort::OneStop;
    QSerialPort::Parity par = QSerialPort::NoParity;

    switch (baudrate) {
        case  9600: br = QSerialPort::Baud9600;
                    break;
        case 57600: br = QSerialPort::Baud57600;
                    break;
        default:    br = QSerialPort::Baud115200;
                    break;
    }

    //TODO Add if needed!
    switch (dataBits) {
        default:    db = QSerialPort::Data8;
                    break;
    }

    switch (stopBits) {
        case  2:    sb = QSerialPort::TwoStop;
                    break;
        default:    sb = QSerialPort::OneStop;
                    break;
    }

    switch (parity) {
        default:    par = QSerialPort::NoParity;
                    break;
    }

    qDebug() << "COM_interface::open(): baudrate = " << baudrate;
    qDebug() << "COM_interface::open(): br = " << br;
    qDebug() << "COM_interface::open(): databits = " << dataBits;
    qDebug() << "COM_interface::open(): db = " << db;
    qDebug() << "COM_interface::open(): stopbits = " << stopBits;
    qDebug() << "COM_interface::open(): sb = " << sb;
    qDebug() << "COM_interface::open(): parity = " << parity;
    qDebug() << "COM_interface::open(): par = " << par;

    port->setPortName(portname);
    port->setBaudRate(br);
    port->setStopBits(sb);
    //port->setFlowControl(FLOW_OFF);
    port->setParity(par);
    port->setDataBits(db);

    if (port->open(QIODevice::ReadWrite) == true) {
        connect(port, SIGNAL(readyRead()), this, SLOT(onReadyRead()));

        qDebug() << "listening for data on" << port->portName();

        emit connected();
        return 1;
    }
    else {
        qDebug() << "device failed to open:";
        qDebug() << "               device:" << portname;
        qDebug() << "          errorstring:" << port->errorString();
        return -1;
    }
}


/***************************************************************************
 * *************************************************************************
 *
 * QT4 ( = QextSerialPort) specific methodes
 *
 ***************************************************************************
 ***************************************************************************/
#else

int COM_interface::open(const QString & portname, int baudrate)
{
    BaudRateType br = BAUD115200;
    switch (baudrate) {
        case  9600: br = BAUD9600;
                    break;
        case 57600: br = BAUD57600;
                    break;
        default:    br = BAUD115200;
                    break;
    }

    qDebug() << "COM_interface::open(): baudrate = " << baudrate;
    qDebug() << "COM_interface::open(): br = " << br;

    port->setPortName(portname);
    port->setBaudRate(br);
    port->setStopBits(STOP_1);
    //port->setFlowControl(FLOW_OFF);
    port->setParity(PAR_NONE);
    port->setDataBits(DATA_8);
    port->setTimeout(0);

    if (port->open(QIODevice::ReadWrite) == true) {
        connect(port, SIGNAL(readyRead()), this, SLOT(onReadyRead()));
        connect(port, SIGNAL(dsrChanged(bool)), this, SLOT(onDsrChanged(bool)));
        if (!(port->lineStatus() & LS_DSR))
            qDebug() << "warning: device is not turned on";
        qDebug() << "listening for data on" << port->portName();

        emit connected();
        return 1;
    }
    else {
        qDebug() << "device failed to open:";
        qDebug() << "               device:" << portname;
        qDebug() << "          errorstring:" << port->errorString();
        return -1;
    }
}

int COM_interface::open(const QString & portname, int baudrate, int dataBits, int stopBits, int parity)
{
    BaudRateType br = BAUD115200;
    DataBitsType db = DATA_8;
    StopBitsType sb = STOP_1;
    ParityType par = PAR_NONE;

    switch (baudrate) {
        case  9600: br = BAUD9600;
                    break;
        case 57600: br = BAUD57600;
                    break;
        default:    br = BAUD115200;
                    break;
    }

    //TODO Add if needed!
    switch (dataBits) {
        default:    db = DATA_8;
                    break;
    }

    switch (stopBits) {
        case  2:    sb = STOP_2;
                    break;
        default:    sb = STOP_1;
                    break;
    }

    switch (parity) {
        default:    par = PAR_NONE;
                    break;
    }

    qDebug() << "COM_interface::open(): baudrate = " << baudrate;
    qDebug() << "COM_interface::open(): br = " << br;
    qDebug() << "COM_interface::open(): databits = " << dataBits;
    qDebug() << "COM_interface::open(): db = " << db;
    qDebug() << "COM_interface::open(): stopbits = " << stopBits;
    qDebug() << "COM_interface::open(): sb = " << sb;
    qDebug() << "COM_interface::open(): parity = " << parity;
    qDebug() << "COM_interface::open(): par = " << par;

    port->setPortName(portname);
    port->setBaudRate(br);
    port->setStopBits(sb);
    //port->setFlowControl(FLOW_OFF);
    port->setParity(par);
    port->setDataBits(db);
    port->setTimeout(0);

    if (port->open(QIODevice::ReadWrite) == true) {
        connect(port, SIGNAL(readyRead()), this, SLOT(onReadyRead()));
        connect(port, SIGNAL(dsrChanged(bool)), this, SLOT(onDsrChanged(bool)));
        if (!(port->lineStatus() & LS_DSR))
            qDebug() << "warning: device is not turned on";
        qDebug() << "listening for data on" << port->portName();

        emit connected();
        return 1;
    }
    else {
        qDebug() << "device failed to open:";
        qDebug() << "               device:" << portname;
        qDebug() << "          errorstring:" << port->errorString();
        return -1;
    }
}
#endif

int COM_interface::close(void)
{
    if (port->isOpen()) {
         port->flush();
         //port->reset();
         port->close();
         port->reset();
    }
    emit disconnected();

    return 0;
}

void COM_interface::onReadyRead(void) {
    qint64 i;
    i = 0;
    buffer->clear();

    while (1) {
      char c;
      if (port->read(& c, 1) <= 0)
         break;
      qDebug() << "com_interface::onReadyRead(): buffer[" << i << "]: '" << c << "'";
      buffer->insert(i++,c);
    }

    emit ReadData(buffer);
}


void COM_interface::onDsrChanged(bool state) {
  // DSR-line is not used with CLR200 reader
  // so we do nothing on this signal.
  qDebug() << "onDsrChanged(): state is " << state;
}


void COM_interface::putChar(const char c) {
  port->putChar(c);
}
